/* Auto-generated public header file */
#ifndef VMHOOK_H
#define VMHOOK_H

#include "omrhookable.h"


/* Begin declarations block */

struct J9BytecodeVerificationData;
struct J9Class;
struct J9ClassLoader;
struct J9ClassPathEntry;
struct J9JavaVM;
struct J9Method;
struct J9Module;
struct J9NativeLibrary;
struct J9ROMClass;
struct J9ROMMethod;
struct J9UTF8;
struct J9VMThread;

typedef void (* classDataFreeFunction)(void * userData, void * address);
typedef UDATA (* lookupNativeAddressCallback)(struct J9VMThread *currentThread, struct J9Method *nativeMethod, struct J9NativeLibrary *handle, char *longJNI, char *shortJNI, UDATA functionArgCount, UDATA bindJNINative);
#include "j9cfg.h"
#include "j9accessbarrier.h"
	
/* End declarations block */

/* J9HOOK_THREAD_ABOUT_TO_START
Triggered as soon as a thread has started, before it has bootstrapped. The thread is not yet ready to run Java code at this point.

Example usage:
	(*hookable)->J9HookRegisterWithCallSite(hookable, J9HOOK_THREAD_ABOUT_TO_START, eventOccurred, OMR_GET_CALLSITE(), NULL);

	static void
	eventOccurred(J9HookInterface** hookInterface, UDATA eventNum, void* voidData, void* userData)
	{
		J9ThreadAboutToStartEvent* eventData = voidData;
		. . .
	}
 */
#define J9HOOK_THREAD_ABOUT_TO_START 1
typedef struct J9ThreadAboutToStartEvent {
	struct J9VMThread* vmThread;
} J9ThreadAboutToStartEvent;

/* J9HOOK_VM_INITIALIZED
Triggered at the end of JNI_CreateJavaVM, just before the start event for the main thread

Example usage:
	(*hookable)->J9HookRegisterWithCallSite(hookable, J9HOOK_VM_INITIALIZED, eventOccurred, OMR_GET_CALLSITE(), NULL);

	static void
	eventOccurred(J9HookInterface** hookInterface, UDATA eventNum, void* voidData, void* userData)
	{
		J9VMInitEvent* eventData = voidData;
		. . .
	}
 */
#define J9HOOK_VM_INITIALIZED 2
typedef struct J9VMInitEvent {
	struct J9VMThread* vmThread;
} J9VMInitEvent;

/* J9HOOK_VM_SHUTTING_DOWN
Triggered by DestroyJavaVM or exit after exit hooks have run, but before the finalizer has shutdown

Example usage:
	(*hookable)->J9HookRegisterWithCallSite(hookable, J9HOOK_VM_SHUTTING_DOWN, eventOccurred, OMR_GET_CALLSITE(), NULL);

	static void
	eventOccurred(J9HookInterface** hookInterface, UDATA eventNum, void* voidData, void* userData)
	{
		J9VMShutdownEvent* eventData = voidData;
		. . .
	}
 */
#define J9HOOK_VM_SHUTTING_DOWN 3
typedef struct J9VMShutdownEvent {
	struct J9VMThread* vmThread;
	IDATA exitCode;
} J9VMShutdownEvent;

/* J9HOOK_VM_THREAD_STARTING

			Triggered when Thread.start() succeeds. The current thread is the caller of Thread.start().
			The current thread has VM access when this hook is fired.
		

Example usage:
	(*hookable)->J9HookRegisterWithCallSite(hookable, J9HOOK_VM_THREAD_STARTING, eventOccurred, OMR_GET_CALLSITE(), NULL);

	static void
	eventOccurred(J9HookInterface** hookInterface, UDATA eventNum, void* voidData, void* userData)
	{
		J9VMThreadStartingEvent* eventData = voidData;
		. . .
	}
 */
#define J9HOOK_VM_THREAD_STARTING 4
typedef struct J9VMThreadStartingEvent {
	struct J9VMThread* currentThread;
	struct J9VMThread* startedThread;
} J9VMThreadStartingEvent;

/* J9HOOK_VM_THREAD_CREATED

			Triggered during initialization of a J9VMThread. At this point the thread has an object associated with it.
			This hook can optionally cause the allocation of the J9VMThread to fail (e.g. if some substructres fail to
			be allocated during the hook.
		

Example usage:
	(*hookable)->J9HookRegisterWithCallSite(hookable, J9HOOK_VM_THREAD_CREATED, eventOccurred, OMR_GET_CALLSITE(), NULL);

	static void
	eventOccurred(J9HookInterface** hookInterface, UDATA eventNum, void* voidData, void* userData)
	{
		J9VMThreadCreatedEvent* eventData = voidData;
		. . .
	}
 */
#define J9HOOK_VM_THREAD_CREATED 5
typedef struct J9VMThreadCreatedEvent {
	struct J9VMThread* vmThread;
	UDATA continueInitialization;
} J9VMThreadCreatedEvent;

/* J9HOOK_VM_THREAD_DESTROY

			Triggered just before freeing a J9VMThread. At this point the thread has an object associated with it.
		

Example usage:
	(*hookable)->J9HookRegisterWithCallSite(hookable, J9HOOK_VM_THREAD_DESTROY, eventOccurred, OMR_GET_CALLSITE(), NULL);

	static void
	eventOccurred(J9HookInterface** hookInterface, UDATA eventNum, void* voidData, void* userData)
	{
		J9VMThreadDestroyEvent* eventData = voidData;
		. . .
	}
 */
#define J9HOOK_VM_THREAD_DESTROY 6
typedef struct J9VMThreadDestroyEvent {
	struct J9VMThread* vmThread;
} J9VMThreadDestroyEvent;

/* J9HOOK_VM_THREAD_STARTED

			Triggered once the thread is ready to run. At this point the thread has an object associated with it.
			Note that this event may be triggered by the main thread (instead of the starting thread) if the
			vmThread was created during bootstrap.  The current thread does not have VM access when this hook
			is fired.
		

Example usage:
	(*hookable)->J9HookRegisterWithCallSite(hookable, J9HOOK_VM_THREAD_STARTED, eventOccurred, OMR_GET_CALLSITE(), NULL);

	static void
	eventOccurred(J9HookInterface** hookInterface, UDATA eventNum, void* voidData, void* userData)
	{
		J9VMThreadStartedEvent* eventData = voidData;
		. . .
	}
 */
#define J9HOOK_VM_THREAD_STARTED 7
typedef struct J9VMThreadStartedEvent {
	struct J9VMThread* currentThread;
	struct J9VMThread* vmThread;
} J9VMThreadStartedEvent;

/* J9HOOK_VM_THREAD_END

			Triggered when a thread is about to die.  The current thread does not have VM access when this hook
			is fired.
		

Example usage:
	(*hookable)->J9HookRegisterWithCallSite(hookable, J9HOOK_VM_THREAD_END, eventOccurred, OMR_GET_CALLSITE(), NULL);

	static void
	eventOccurred(J9HookInterface** hookInterface, UDATA eventNum, void* voidData, void* userData)
	{
		J9VMThreadEndEvent* eventData = voidData;
		. . .
	}
 */
#define J9HOOK_VM_THREAD_END 8
typedef struct J9VMThreadEndEvent {
	struct J9VMThread* currentThread;
	UDATA destroyingJavaVM;
} J9VMThreadEndEvent;

/* J9HOOK_VM_CLASS_LOAD

				Triggered immediately after a RAM class has been created for a newly loaded class.
				Static final fields have been initialized to their default values (zero).
				VM access is released. It is safe to execute Java code from handlers of this hook. If
				an exception is pending on return, the class load fails.
		

Example usage:
	(*hookable)->J9HookRegisterWithCallSite(hookable, J9HOOK_VM_CLASS_LOAD, eventOccurred, OMR_GET_CALLSITE(), NULL);

	static void
	eventOccurred(J9HookInterface** hookInterface, UDATA eventNum, void* voidData, void* userData)
	{
		J9VMClassLoadEvent* eventData = voidData;
		. . .
	}
 */
#define J9HOOK_VM_CLASS_LOAD 9
typedef struct J9VMClassLoadEvent {
	struct J9VMThread* currentThread;
	struct J9Class* clazz;
} J9VMClassLoadEvent;

/* J9HOOK_VM_EXCEPTION_THROW

				Triggered when an exception is about to be thrown. The thread's currentException
				field has been cleared at this point.
				VM access is held by the current thread. The handler may release VM access, but
				if it does so, it is the handler's responsibility to save the exception object and update
				the 'exception' field of the event structure with the new address of the exception.
		

Example usage:
	(*hookable)->J9HookRegisterWithCallSite(hookable, J9HOOK_VM_EXCEPTION_THROW, eventOccurred, OMR_GET_CALLSITE(), NULL);

	static void
	eventOccurred(J9HookInterface** hookInterface, UDATA eventNum, void* voidData, void* userData)
	{
		J9VMExceptionThrowEvent* eventData = voidData;
		. . .
	}
 */
#define J9HOOK_VM_EXCEPTION_THROW 10
typedef struct J9VMExceptionThrowEvent {
	struct J9VMThread* currentThread;
	j9object_t exception;
} J9VMExceptionThrowEvent;

/* J9HOOK_VM_EXCEPTION_SYSTHROW

				Triggered when an exception is about to be thrown by the VM. The thread's currentException
				field has been cleared at this point.
				VM access is held by the current thread. The handler may release VM access, but
				if it does so, it is the handler's responsibility to save the exception object and update
				the 'exception' field of the event structure with the new address of the exception.
		

Example usage:
	(*hookable)->J9HookRegisterWithCallSite(hookable, J9HOOK_VM_EXCEPTION_SYSTHROW, eventOccurred, OMR_GET_CALLSITE(), NULL);

	static void
	eventOccurred(J9HookInterface** hookInterface, UDATA eventNum, void* voidData, void* userData)
	{
		J9VMExceptionSysThrowEvent* eventData = voidData;
		. . .
	}
 */
#define J9HOOK_VM_EXCEPTION_SYSTHROW 11
typedef struct J9VMExceptionSysThrowEvent {
	struct J9VMThread* currentThread;
	j9object_t exception;
} J9VMExceptionSysThrowEvent;

/* J9HOOK_VM_EXCEPTION_CATCH

				Triggered when an exception is about to be caught.
				VM access is held by the current thread. The handler may release VM access, but
				if it does so, it is the handler's responsibility to save the exception object and update
				the 'exception' field of the event structure with the new address of the exception.
		

Example usage:
	(*hookable)->J9HookRegisterWithCallSite(hookable, J9HOOK_VM_EXCEPTION_CATCH, eventOccurred, OMR_GET_CALLSITE(), NULL);

	static void
	eventOccurred(J9HookInterface** hookInterface, UDATA eventNum, void* voidData, void* userData)
	{
		J9VMExceptionCatchEvent* eventData = voidData;
		. . .
	}
 */
#define J9HOOK_VM_EXCEPTION_CATCH 12
typedef struct J9VMExceptionCatchEvent {
	struct J9VMThread* currentThread;
	j9object_t exception;
	UDATA* popProtectedFrame;
} J9VMExceptionCatchEvent;

/* J9HOOK_VM_BREAKPOINT

				Triggered when a breakpointed bytecode is executed.
				The handler must pass back the true bytecode to execute in the originalBytecode field.
				The current thread has VM access. VM access may be temporarily released by a handler.
		

Example usage:
	(*hookable)->J9HookRegisterWithCallSite(hookable, J9HOOK_VM_BREAKPOINT, eventOccurred, OMR_GET_CALLSITE(), NULL);

	static void
	eventOccurred(J9HookInterface** hookInterface, UDATA eventNum, void* voidData, void* userData)
	{
		J9VMBreakpointEvent* eventData = voidData;
		. . .
	}
 */
#define J9HOOK_VM_BREAKPOINT 13
typedef struct J9VMBreakpointEvent {
	struct J9VMThread* currentThread;
	struct J9Method* method;
	IDATA location;
	UDATA originalBytecode;
} J9VMBreakpointEvent;

/* J9HOOK_VM_FRAME_POPPED

				Triggered when a pop-protected frame is popped from the stack.
				This event is only supported if J9VM_INTERP_DEBUG_SUPPORT is defined.
		

Example usage:
	(*hookable)->J9HookRegisterWithCallSite(hookable, J9HOOK_VM_FRAME_POPPED, eventOccurred, OMR_GET_CALLSITE(), NULL);

	static void
	eventOccurred(J9HookInterface** hookInterface, UDATA eventNum, void* voidData, void* userData)
	{
		J9VMFramePoppedEvent* eventData = voidData;
		. . .
	}
 */
#define J9HOOK_VM_FRAME_POPPED 14
typedef struct J9VMFramePoppedEvent {
	struct J9VMThread* currentThread;
} J9VMFramePoppedEvent;

/* J9HOOK_VM_MONITOR_CONTENDED_ENTER

				Triggered when a thread is about to block on a Java monitor.
				The current thread has VM access, but a handler may safely release and reacquire access.
		

Example usage:
	(*hookable)->J9HookRegisterWithCallSite(hookable, J9HOOK_VM_MONITOR_CONTENDED_ENTER, eventOccurred, OMR_GET_CALLSITE(), NULL);

	static void
	eventOccurred(J9HookInterface** hookInterface, UDATA eventNum, void* voidData, void* userData)
	{
		J9VMMonitorContendedEnterEvent* eventData = voidData;
		. . .
	}
 */
#define J9HOOK_VM_MONITOR_CONTENDED_ENTER 15
typedef struct J9VMMonitorContendedEnterEvent {
	struct J9VMThread* currentThread;
	omrthread_monitor_t monitor;
} J9VMMonitorContendedEnterEvent;

/* J9HOOK_VM_MONITOR_CONTENDED_ENTERED

				Triggered when a thread has finished blocking on a Java monitor.
				The current thread has VM access, but a handler may safely release and reacquire access.
		

Example usage:
	(*hookable)->J9HookRegisterWithCallSite(hookable, J9HOOK_VM_MONITOR_CONTENDED_ENTERED, eventOccurred, OMR_GET_CALLSITE(), NULL);

	static void
	eventOccurred(J9HookInterface** hookInterface, UDATA eventNum, void* voidData, void* userData)
	{
		J9VMMonitorContendedEnteredEvent* eventData = voidData;
		. . .
	}
 */
#define J9HOOK_VM_MONITOR_CONTENDED_ENTERED 16
typedef struct J9VMMonitorContendedEnteredEvent {
	struct J9VMThread* currentThread;
	omrthread_monitor_t monitor;
	I_64 startTicks;
	struct J9Class* monitorClass;
	struct J9VMThread* previousOwner;
} J9VMMonitorContendedEnteredEvent;

/* J9HOOK_VM_EXCEPTION_DESCRIBE

				Triggered when an exception is about to be reported as uncaught.
				VM access is held by the current thread. The handler may release VM access, but if it
				does so it must update the 'exception' field in the event data for subsequent handlers, as
				the object may have moved.
		

Example usage:
	(*hookable)->J9HookRegisterWithCallSite(hookable, J9HOOK_VM_EXCEPTION_DESCRIBE, eventOccurred, OMR_GET_CALLSITE(), NULL);

	static void
	eventOccurred(J9HookInterface** hookInterface, UDATA eventNum, void* voidData, void* userData)
	{
		J9VMExceptionDescribeEvent* eventData = voidData;
		. . .
	}
 */
#define J9HOOK_VM_EXCEPTION_DESCRIBE 17
typedef struct J9VMExceptionDescribeEvent {
	struct J9VMThread* currentThread;
	j9object_t exception;
} J9VMExceptionDescribeEvent;

/* J9HOOK_VM_GETENV

				Triggered when JNI GetEnv is called.
				VM access is not held by the current thread.
		

Example usage:
	(*hookable)->J9HookRegisterWithCallSite(hookable, J9HOOK_VM_GETENV, eventOccurred, OMR_GET_CALLSITE(), NULL);

	static void
	eventOccurred(J9HookInterface** hookInterface, UDATA eventNum, void* voidData, void* userData)
	{
		J9VMGetEnvEvent* eventData = voidData;
		. . .
	}
 */
#define J9HOOK_VM_GETENV 18
typedef struct J9VMGetEnvEvent {
	JavaVM* jvm;
	void** penv;
	jint version;
	jint rc;
} J9VMGetEnvEvent;

/* J9HOOK_VM_SLOW_EXCLUSIVE

				Triggered when a thread takes longer than a specified time to respond to an exclusive access request.
				It is not safe to execute Java code from within a handler for this event, as the current thread owns the
				exclusiveAccessMutex.
		

Example usage:
	(*hookable)->J9HookRegisterWithCallSite(hookable, J9HOOK_VM_SLOW_EXCLUSIVE, eventOccurred, OMR_GET_CALLSITE(), NULL);

	static void
	eventOccurred(J9HookInterface** hookInterface, UDATA eventNum, void* voidData, void* userData)
	{
		J9VMSlowExclusiveEvent* eventData = voidData;
		. . .
	}
 */
#define J9HOOK_VM_SLOW_EXCLUSIVE 19
typedef struct J9VMSlowExclusiveEvent {
	struct J9VMThread* currentThread;
	UDATA timeTaken;
	UDATA reason;
} J9VMSlowExclusiveEvent;

/* J9HOOK_VM_ACQUIREVMACCESS

				Triggered when a thread is about to acquire VM access.
		

Example usage:
	(*hookable)->J9HookRegisterWithCallSite(hookable, J9HOOK_VM_ACQUIREVMACCESS, eventOccurred, OMR_GET_CALLSITE(), NULL);

	static void
	eventOccurred(J9HookInterface** hookInterface, UDATA eventNum, void* voidData, void* userData)
	{
		J9VMAcquireVMAccessEvent* eventData = voidData;
		. . .
	}
 */
#define J9HOOK_VM_ACQUIREVMACCESS 20
typedef struct J9VMAcquireVMAccessEvent {
	struct J9VMThread* currentThread;
} J9VMAcquireVMAccessEvent;

/* J9HOOK_VM_RELEASEVMACCESS

				Triggered after a thread releases VM access.
		

Example usage:
	(*hookable)->J9HookRegisterWithCallSite(hookable, J9HOOK_VM_RELEASEVMACCESS, eventOccurred, OMR_GET_CALLSITE(), NULL);

	static void
	eventOccurred(J9HookInterface** hookInterface, UDATA eventNum, void* voidData, void* userData)
	{
		J9VMReleaseVMAccessEvent* eventData = voidData;
		. . .
	}
 */
#define J9HOOK_VM_RELEASEVMACCESS 21
typedef struct J9VMReleaseVMAccessEvent {
	struct J9VMThread* currentThread;
} J9VMReleaseVMAccessEvent;

/* J9HOOK_VM_ACQUIRING_EXCLUSIVE_IN_NATIVE

				Triggered during acquire exclusive thread loop, when encountered a thread in native (which we did not count for exclusive).
				This hook serves similar purpose as J9HOOK_VM_RELEASEVMACCESS, to ensure that required work for thread in native
				is performed before exclusive access is obtained.
		

Example usage:
	(*hookable)->J9HookRegisterWithCallSite(hookable, J9HOOK_VM_ACQUIRING_EXCLUSIVE_IN_NATIVE, eventOccurred, OMR_GET_CALLSITE(), NULL);

	static void
	eventOccurred(J9HookInterface** hookInterface, UDATA eventNum, void* voidData, void* userData)
	{
		J9VMAcquringExclusiveInNativeEvent* eventData = voidData;
		. . .
	}
 */
#define J9HOOK_VM_ACQUIRING_EXCLUSIVE_IN_NATIVE 22
typedef struct J9VMAcquringExclusiveInNativeEvent {
	struct J9VMThread* currentThread;
	struct J9VMThread* targetThread;
} J9VMAcquringExclusiveInNativeEvent;

/* J9HOOK_VM_THREAD_CRASH

				Triggered when a thread has crashed. The VM is about to terminate, and may be in an unstable state
		

Example usage:
	(*hookable)->J9HookRegisterWithCallSite(hookable, J9HOOK_VM_THREAD_CRASH, eventOccurred, OMR_GET_CALLSITE(), NULL);

	static void
	eventOccurred(J9HookInterface** hookInterface, UDATA eventNum, void* voidData, void* userData)
	{
		J9VMThreadCrashEvent* eventData = voidData;
		. . .
	}
 */
#define J9HOOK_VM_THREAD_CRASH 23
typedef struct J9VMThreadCrashEvent {
	struct J9VMThread* currentThread;
} J9VMThreadCrashEvent;

/* J9HOOK_VM_METHOD_ENTER

			Triggered every time a method is entered. Hooking this event may have a significant performance impact.
			Even reserving this event will cause the VM and JIT to run with reduced optimization.
			When this event is reported, the current thread has VM access.
		

Example usage:
	(*hookable)->J9HookRegisterWithCallSite(hookable, J9HOOK_VM_METHOD_ENTER, eventOccurred, OMR_GET_CALLSITE(), NULL);

	static void
	eventOccurred(J9HookInterface** hookInterface, UDATA eventNum, void* voidData, void* userData)
	{
		J9VMMethodEnterEvent* eventData = voidData;
		. . .
	}
 */
#define J9HOOK_VM_METHOD_ENTER 24
typedef struct J9VMMethodEnterEvent {
	struct J9VMThread* currentThread;
	struct J9Method* method;
	void* arg0EA;
	UDATA methodType;
} J9VMMethodEnterEvent;

/* J9HOOK_VM_ABOUT_TO_BOOTSTRAP

			Triggered just before the VM starts loading classes and executing bytecodes.
		

Example usage:
	(*hookable)->J9HookRegisterWithCallSite(hookable, J9HOOK_VM_ABOUT_TO_BOOTSTRAP, eventOccurred, OMR_GET_CALLSITE(), NULL);

	static void
	eventOccurred(J9HookInterface** hookInterface, UDATA eventNum, void* voidData, void* userData)
	{
		J9VMAboutToBootstrapEvent* eventData = voidData;
		. . .
	}
 */
#define J9HOOK_VM_ABOUT_TO_BOOTSTRAP 25
typedef struct J9VMAboutToBootstrapEvent {
	struct J9VMThread* currentThread;
} J9VMAboutToBootstrapEvent;

/* J9HOOK_VM_METHOD_RETURN

			Triggered every time a method is about to return. Hooking this event may have a significant performance impact.
			Even reserving this event will cause the VM and JIT to run with reduced optimization.
			When this event is reported, the current thread has VM access.  If the return value is an object, it will be
			kept up-to-date by the caller.
		

Example usage:
	(*hookable)->J9HookRegisterWithCallSite(hookable, J9HOOK_VM_METHOD_RETURN, eventOccurred, OMR_GET_CALLSITE(), NULL);

	static void
	eventOccurred(J9HookInterface** hookInterface, UDATA eventNum, void* voidData, void* userData)
	{
		J9VMMethodReturnEvent* eventData = voidData;
		. . .
	}
 */
#define J9HOOK_VM_METHOD_RETURN 26
typedef struct J9VMMethodReturnEvent {
	struct J9VMThread* currentThread;
	struct J9Method* method;
	UDATA poppedByException;
	void* returnValuePtr;
	UDATA methodType;
} J9VMMethodReturnEvent;

/* J9HOOK_VM_NATIVE_METHOD_ENTER

			Triggered every time a native method is entered.
			When this event is reported, the current thread has VM access.
		

Example usage:
	(*hookable)->J9HookRegisterWithCallSite(hookable, J9HOOK_VM_NATIVE_METHOD_ENTER, eventOccurred, OMR_GET_CALLSITE(), NULL);

	static void
	eventOccurred(J9HookInterface** hookInterface, UDATA eventNum, void* voidData, void* userData)
	{
		J9VMNativeMethodEnterEvent* eventData = voidData;
		. . .
	}
 */
#define J9HOOK_VM_NATIVE_METHOD_ENTER 27
typedef struct J9VMNativeMethodEnterEvent {
	struct J9VMThread* currentThread;
	struct J9Method* method;
	void* arg0EA;
} J9VMNativeMethodEnterEvent;

/* J9HOOK_VM_NATIVE_METHOD_RETURN

			Triggered every time a native method is about to return.
			Reserving this event will cause the JIT to run with reduced optimization.
			When this event is reported, the current thread has VM access.  If the return value is an object, it will be
			kept up-to-date by the caller.
		

Example usage:
	(*hookable)->J9HookRegisterWithCallSite(hookable, J9HOOK_VM_NATIVE_METHOD_RETURN, eventOccurred, OMR_GET_CALLSITE(), NULL);

	static void
	eventOccurred(J9HookInterface** hookInterface, UDATA eventNum, void* voidData, void* userData)
	{
		J9VMNativeMethodReturnEvent* eventData = voidData;
		. . .
	}
 */
#define J9HOOK_VM_NATIVE_METHOD_RETURN 28
typedef struct J9VMNativeMethodReturnEvent {
	struct J9VMThread* currentThread;
	struct J9Method* method;
	UDATA poppedByException;
	void* returnValuePtr;
	jobject returnRef;
} J9VMNativeMethodReturnEvent;

/* J9HOOK_VM_ROM_CLASS_LOAD

				Triggered immediately after a ROM class has been created for a newly loaded class.
				(Does this belong in the VM hook interface, or should dynload have its own interface?)
		

Example usage:
	(*hookable)->J9HookRegisterWithCallSite(hookable, J9HOOK_VM_ROM_CLASS_LOAD, eventOccurred, OMR_GET_CALLSITE(), NULL);

	static void
	eventOccurred(J9HookInterface** hookInterface, UDATA eventNum, void* voidData, void* userData)
	{
		J9VMROMClassLoadEvent* eventData = voidData;
		. . .
	}
 */
#define J9HOOK_VM_ROM_CLASS_LOAD 29
typedef struct J9VMROMClassLoadEvent {
	struct J9VMThread* currentThread;
	struct J9ROMClass* romClass;
} J9VMROMClassLoadEvent;

/* J9HOOK_VM_MONITOR_CONTENDED_EXIT

				Triggered when a thread is about to exit a Java monitor on which other threads are contending.
				The current thread has VM access, but a handler may safely release and reacquire access.
		

Example usage:
	(*hookable)->J9HookRegisterWithCallSite(hookable, J9HOOK_VM_MONITOR_CONTENDED_EXIT, eventOccurred, OMR_GET_CALLSITE(), NULL);

	static void
	eventOccurred(J9HookInterface** hookInterface, UDATA eventNum, void* voidData, void* userData)
	{
		J9VMMonitorContendedExitEvent* eventData = voidData;
		. . .
	}
 */
#define J9HOOK_VM_MONITOR_CONTENDED_EXIT 30
typedef struct J9VMMonitorContendedExitEvent {
	struct J9VMThread* currentThread;
	omrthread_monitor_t monitor;
} J9VMMonitorContendedExitEvent;

/* J9HOOK_VM_CLASS_LOAD_HOOK

				Triggered just before a class file is transformed into a ROM class. Listeners of this event
				are permitted to replace the class file data with new data, allowing instrumentation to take
				place. If new data is provided, the old data should be freed using the freeFunction.  If the
				freeFunction is NULL, nothing need be done.  If not, it should be called with the freeUserData and
				classData fields from this struct.  The freeUserData and freeFunction must be updated such that
				the new classData can be freed in similar fashion by another listener or the sender of this
				event.
				The current thread has VM access and owns the class table mutex.
				If VM access is released, the listener is reponsible for keeping the protectionDomain object up-to-date.
		

Example usage:
	(*hookable)->J9HookRegisterWithCallSite(hookable, J9HOOK_VM_CLASS_LOAD_HOOK, eventOccurred, OMR_GET_CALLSITE(), NULL);

	static void
	eventOccurred(J9HookInterface** hookInterface, UDATA eventNum, void* voidData, void* userData)
	{
		J9VMClassLoadHookEvent* eventData = voidData;
		. . .
	}
 */
#define J9HOOK_VM_CLASS_LOAD_HOOK 31
typedef struct J9VMClassLoadHookEvent {
	struct J9VMThread* currentThread;
	struct J9ClassLoader* classLoader;
	j9object_t protectionDomain;
	struct J9Class* oldClass;
	char* className;
	U_8* classData;
	UDATA classDataLength;
	void* freeUserData;
	classDataFreeFunction freeFunction;
	UDATA classDataReplaced;
} J9VMClassLoadHookEvent;

/* J9HOOK_VM_CLASS_LOAD_HOOK2

				See J9HOOK_VM_CLASS_LOAD_HOOK.  The struct must be identical.
				This event is used for JVMTI 1.1 RetransformClasses.
		

Example usage:
	(*hookable)->J9HookRegisterWithCallSite(hookable, J9HOOK_VM_CLASS_LOAD_HOOK2, eventOccurred, OMR_GET_CALLSITE(), NULL);

	static void
	eventOccurred(J9HookInterface** hookInterface, UDATA eventNum, void* voidData, void* userData)
	{
		J9VMClassLoadHook2Event* eventData = voidData;
		. . .
	}
 */
#define J9HOOK_VM_CLASS_LOAD_HOOK2 32
typedef struct J9VMClassLoadHook2Event {
	struct J9VMThread* currentThread;
	struct J9ClassLoader* classLoader;
	j9object_t protectionDomain;
	struct J9Class* oldClass;
	char* className;
	U_8* classData;
	UDATA classDataLength;
	void* freeUserData;
	classDataFreeFunction freeFunction;
	UDATA classDataReplaced;
} J9VMClassLoadHook2Event;

/* J9HOOK_VM_OBJECT_ALLOCATE

				Triggered when an object has just been allocated in a non-instrumentable fashion.  See JVMTI spec for details.
				This event is included in the VM hook interface (rather than memory manager interface), since in-line allocates are so common.
				The current thread has VM access, but a handler may safely release and reacquire access.  If VM access is released,
				the listener is responsible for keeping the object up-to-date.
		

Example usage:
	(*hookable)->J9HookRegisterWithCallSite(hookable, J9HOOK_VM_OBJECT_ALLOCATE, eventOccurred, OMR_GET_CALLSITE(), NULL);

	static void
	eventOccurred(J9HookInterface** hookInterface, UDATA eventNum, void* voidData, void* userData)
	{
		J9VMObjectAllocateEvent* eventData = voidData;
		. . .
	}
 */
#define J9HOOK_VM_OBJECT_ALLOCATE 33
typedef struct J9VMObjectAllocateEvent {
	struct J9VMThread* currentThread;
	j9object_t object;
	UDATA size;
} J9VMObjectAllocateEvent;

/* J9HOOK_VM_OBJECT_ALLOCATE_INSTRUMENTABLE

				Triggered when an object has just been allocated, but could be instrumented to provide notification.
				This event is included in the VM hook interface (rather than memory manager interface), since in-line allocates are so common.
				The current thread has VM access, but a handler may safely release and reacquire access.  If VM access is released,
				the listener is responsible for keeping the object up-to-date.
		

Example usage:
	(*hookable)->J9HookRegisterWithCallSite(hookable, J9HOOK_VM_OBJECT_ALLOCATE_INSTRUMENTABLE, eventOccurred, OMR_GET_CALLSITE(), NULL);

	static void
	eventOccurred(J9HookInterface** hookInterface, UDATA eventNum, void* voidData, void* userData)
	{
		J9VMObjectAllocateInstrumentableEvent* eventData = voidData;
		. . .
	}
 */
#define J9HOOK_VM_OBJECT_ALLOCATE_INSTRUMENTABLE 34
typedef struct J9VMObjectAllocateInstrumentableEvent {
	struct J9VMThread* currentThread;
	j9object_t object;
	UDATA size;
} J9VMObjectAllocateInstrumentableEvent;

/* J9HOOK_VM_USER_INTERRUPT
Triggered when a user sends an interrupt signal to the VM

Example usage:
	(*hookable)->J9HookRegisterWithCallSite(hookable, J9HOOK_VM_USER_INTERRUPT, eventOccurred, OMR_GET_CALLSITE(), NULL);

	static void
	eventOccurred(J9HookInterface** hookInterface, UDATA eventNum, void* voidData, void* userData)
	{
		J9VMUserInterruptEvent* eventData = voidData;
		. . .
	}
 */
#define J9HOOK_VM_USER_INTERRUPT 35
typedef struct J9VMUserInterruptEvent {
	struct J9JavaVM* vm;
} J9VMUserInterruptEvent;

/* J9HOOK_VM_INITIALIZE_SEND_TARGET

				Triggered when a RAM method is having its send target address initialized.
				The send target (methodRunAddress) of the method is NULL at the beginning of this event. If a listener sets methodRunAddress
				to another value, then further send target initialization is skipped, and the listener's value is used.
		

Example usage:
	(*hookable)->J9HookRegisterWithCallSite(hookable, J9HOOK_VM_INITIALIZE_SEND_TARGET, eventOccurred, OMR_GET_CALLSITE(), NULL);

	static void
	eventOccurred(J9HookInterface** hookInterface, UDATA eventNum, void* voidData, void* userData)
	{
		J9VMInitializeSendTargetEvent* eventData = voidData;
		. . .
	}
 */
#define J9HOOK_VM_INITIALIZE_SEND_TARGET 36
typedef struct J9VMInitializeSendTargetEvent {
	struct J9VMThread* currentThread;
	struct J9Method* method;
} J9VMInitializeSendTargetEvent;

/* J9HOOK_VM_MONITOR_WAIT

				Triggered when a thread is about to wait on a Java monitor.
				The current thread has VM access, but a handler may safely release and reacquire access.
				Note that this event is reported before all error conditions have been checked. The wait may
				terminate abnormally due to an IllegalMonitorStateException, for instance.
		

Example usage:
	(*hookable)->J9HookRegisterWithCallSite(hookable, J9HOOK_VM_MONITOR_WAIT, eventOccurred, OMR_GET_CALLSITE(), NULL);

	static void
	eventOccurred(J9HookInterface** hookInterface, UDATA eventNum, void* voidData, void* userData)
	{
		J9VMMonitorWaitEvent* eventData = voidData;
		. . .
	}
 */
#define J9HOOK_VM_MONITOR_WAIT 37
typedef struct J9VMMonitorWaitEvent {
	struct J9VMThread* currentThread;
	omrthread_monitor_t monitor;
	I_64 millis;
	I_32 nanos;
} J9VMMonitorWaitEvent;

/* J9HOOK_VM_MONITOR_WAITED

				Triggered when a thread is has finished waiting on a Java monitor.
				The current thread has VM access, but a handler may safely release and reacquire access.
		

Example usage:
	(*hookable)->J9HookRegisterWithCallSite(hookable, J9HOOK_VM_MONITOR_WAITED, eventOccurred, OMR_GET_CALLSITE(), NULL);

	static void
	eventOccurred(J9HookInterface** hookInterface, UDATA eventNum, void* voidData, void* userData)
	{
		J9VMMonitorWaitedEvent* eventData = voidData;
		. . .
	}
 */
#define J9HOOK_VM_MONITOR_WAITED 38
typedef struct J9VMMonitorWaitedEvent {
	struct J9VMThread* currentThread;
	omrthread_monitor_t monitor;
	I_64 millis;
	I_32 nanos;
	IDATA reason;
	I_64 startTicks;
	UDATA monitorAddress;
	struct J9Class* monitorClass;
} J9VMMonitorWaitedEvent;

/* J9HOOK_VM_STARTED
Triggered once enough initialization has taken place to use JNI

Example usage:
	(*hookable)->J9HookRegisterWithCallSite(hookable, J9HOOK_VM_STARTED, eventOccurred, OMR_GET_CALLSITE(), NULL);

	static void
	eventOccurred(J9HookInterface** hookInterface, UDATA eventNum, void* voidData, void* userData)
	{
		J9VMStartEvent* eventData = voidData;
		. . .
	}
 */
#define J9HOOK_VM_STARTED 39
typedef struct J9VMStartEvent {
	struct J9VMThread* vmThread;
} J9VMStartEvent;

/* J9HOOK_VM_REQUIRED_DEBUG_ATTRIBUTES

			Triggered exactly once before any classes are dynamically loaded, then disabled.
			The consumer must bitwise OR any required debug attribute bits into requiredDebugAttributes.
			Valid attributes are the J9VM_DEBUG_ATTRIBUTE bits in j9.h.
			Currently only triggered if J9VM_OPT_DEBUG_INFO_SERVER is defined.  If it is not defined,
			the hook will be disabled immediately to indicate that no debug attributes will be stored.
		

Example usage:
	(*hookable)->J9HookRegisterWithCallSite(hookable, J9HOOK_VM_REQUIRED_DEBUG_ATTRIBUTES, eventOccurred, OMR_GET_CALLSITE(), NULL);

	static void
	eventOccurred(J9HookInterface** hookInterface, UDATA eventNum, void* voidData, void* userData)
	{
		J9VMRequiredDebugAttributesEvent* eventData = voidData;
		. . .
	}
 */
#define J9HOOK_VM_REQUIRED_DEBUG_ATTRIBUTES 40
typedef struct J9VMRequiredDebugAttributesEvent {
	struct J9VMThread* vmThread;
	UDATA requiredDebugAttributes;
} J9VMRequiredDebugAttributesEvent;

/* J9HOOK_VM_CLASS_PREPARE

				Triggered after a class is prepared.  This event is not sent for primitive or array classes.
				Static final fields have been initialized to their initial values from the class file.
				VM access is released. It is safe to execute Java code from handlers of this hook. If
				an exception is pending on return, the class load fails.
		

Example usage:
	(*hookable)->J9HookRegisterWithCallSite(hookable, J9HOOK_VM_CLASS_PREPARE, eventOccurred, OMR_GET_CALLSITE(), NULL);

	static void
	eventOccurred(J9HookInterface** hookInterface, UDATA eventNum, void* voidData, void* userData)
	{
		J9VMClassPrepareEvent* eventData = voidData;
		. . .
	}
 */
#define J9HOOK_VM_CLASS_PREPARE 41
typedef struct J9VMClassPrepareEvent {
	struct J9VMThread* currentThread;
	struct J9Class* clazz;
} J9VMClassPrepareEvent;

/* J9HOOK_VM_FIND_METHOD_FROM_PC

			Triggered when the VM searches for a method from a PC (program counter) and fails to find the method.
			This gives listeners an opportunity to provide the method which the VM was unable to find on its own.
		

Example usage:
	(*hookable)->J9HookRegisterWithCallSite(hookable, J9HOOK_VM_FIND_METHOD_FROM_PC, eventOccurred, OMR_GET_CALLSITE(), NULL);

	static void
	eventOccurred(J9HookInterface** hookInterface, UDATA eventNum, void* voidData, void* userData)
	{
		J9VMFindMethodFromPCEvent* eventData = voidData;
		. . .
	}
 */
#define J9HOOK_VM_FIND_METHOD_FROM_PC 42
typedef struct J9VMFindMethodFromPCEvent {
	struct J9VMThread* currentThread;
	U_8* pc;
	struct J9Class* clazz;
	struct J9Method* result;
} J9VMFindMethodFromPCEvent;

/* J9HOOK_VM_PERMANENT_PC

			When a method PC is about to be recorded in, for instance, an exception trace, this hook is triggered
			to give listeners an opportunity to substitute a different PC. Generally, this is used if the PC points to
			a transient data area, such as a breakpointed method. In that case, the listener replaces the PC with
			the equivalent PC in the original ROM method.
		

Example usage:
	(*hookable)->J9HookRegisterWithCallSite(hookable, J9HOOK_VM_PERMANENT_PC, eventOccurred, OMR_GET_CALLSITE(), NULL);

	static void
	eventOccurred(J9HookInterface** hookInterface, UDATA eventNum, void* voidData, void* userData)
	{
		J9VMPermanentPCEvent* eventData = voidData;
		. . .
	}
 */
#define J9HOOK_VM_PERMANENT_PC 43
typedef struct J9VMPermanentPCEvent {
	struct J9VMThread* currentThread;
	U_8* pc;
} J9VMPermanentPCEvent;

/* J9HOOK_VM_PARK

			Triggered when a thread is about to park.
		

Example usage:
	(*hookable)->J9HookRegisterWithCallSite(hookable, J9HOOK_VM_PARK, eventOccurred, OMR_GET_CALLSITE(), NULL);

	static void
	eventOccurred(J9HookInterface** hookInterface, UDATA eventNum, void* voidData, void* userData)
	{
		J9VMParkEvent* eventData = voidData;
		. . .
	}
 */
#define J9HOOK_VM_PARK 44
typedef struct J9VMParkEvent {
	struct J9VMThread * currentThread;
	I_64 millis;
	IDATA nanos;
} J9VMParkEvent;

/* J9HOOK_VM_UNPARKED

			Triggered when a thread resumes from being parked.
		

Example usage:
	(*hookable)->J9HookRegisterWithCallSite(hookable, J9HOOK_VM_UNPARKED, eventOccurred, OMR_GET_CALLSITE(), NULL);

	static void
	eventOccurred(J9HookInterface** hookInterface, UDATA eventNum, void* voidData, void* userData)
	{
		J9VMUnparkedEvent* eventData = voidData;
		. . .
	}
 */
#define J9HOOK_VM_UNPARKED 45
typedef struct J9VMUnparkedEvent {
	struct J9VMThread * currentThread;
	I_64 millis;
	IDATA nanos;
	I_64 startTicks;
	UDATA parkedAddress;
	struct J9Class* parkedClass;
} J9VMUnparkedEvent;

/* J9HOOK_VM_GET_FIELD

			Triggered just before an instance field is read by the bytecode interpreter
			or JNI.  If the listener allows a GC to occur, the object field in the hook
			data must be kept up-to-date.
		

Example usage:
	(*hookable)->J9HookRegisterWithCallSite(hookable, J9HOOK_VM_GET_FIELD, eventOccurred, OMR_GET_CALLSITE(), NULL);

	static void
	eventOccurred(J9HookInterface** hookInterface, UDATA eventNum, void* voidData, void* userData)
	{
		J9VMGetFieldEvent* eventData = voidData;
		. . .
	}
 */
#define J9HOOK_VM_GET_FIELD 46
typedef struct J9VMGetFieldEvent {
	struct J9VMThread* currentThread;
	struct J9Method* method;
	IDATA location;
	j9object_t object;
	UDATA offset;
} J9VMGetFieldEvent;

/* J9HOOK_VM_PUT_FIELD

			Triggered just before an instance field is written by the bytecode interpreter
			or JNI.  If the listener allows a GC to occur, the object field in the hook
			data must be kept up-to-date.  Similarly for the contents of newValue if it
			represents an object.
		

Example usage:
	(*hookable)->J9HookRegisterWithCallSite(hookable, J9HOOK_VM_PUT_FIELD, eventOccurred, OMR_GET_CALLSITE(), NULL);

	static void
	eventOccurred(J9HookInterface** hookInterface, UDATA eventNum, void* voidData, void* userData)
	{
		J9VMPutFieldEvent* eventData = voidData;
		. . .
	}
 */
#define J9HOOK_VM_PUT_FIELD 47
typedef struct J9VMPutFieldEvent {
	struct J9VMThread* currentThread;
	struct J9Method* method;
	IDATA location;
	j9object_t object;
	UDATA offset;
	U_64 newValue;
} J9VMPutFieldEvent;

/* J9HOOK_VM_GET_STATIC_FIELD

			Triggered just before a static field is read by the bytecode interpreter
			or JNI.
		

Example usage:
	(*hookable)->J9HookRegisterWithCallSite(hookable, J9HOOK_VM_GET_STATIC_FIELD, eventOccurred, OMR_GET_CALLSITE(), NULL);

	static void
	eventOccurred(J9HookInterface** hookInterface, UDATA eventNum, void* voidData, void* userData)
	{
		J9VMGetStaticFieldEvent* eventData = voidData;
		. . .
	}
 */
#define J9HOOK_VM_GET_STATIC_FIELD 48
typedef struct J9VMGetStaticFieldEvent {
	struct J9VMThread* currentThread;
	struct J9Method* method;
	IDATA location;
	struct J9Class* declaringClass;
	void* fieldAddress;
} J9VMGetStaticFieldEvent;

/* J9HOOK_VM_PUT_STATIC_FIELD

			Triggered just before a static field is written by the bytecode interpreter
			or JNI. If the listener allows a GC to occur and the contents of newValue
			represents an object, the contents must be kept up-to-date.
		

Example usage:
	(*hookable)->J9HookRegisterWithCallSite(hookable, J9HOOK_VM_PUT_STATIC_FIELD, eventOccurred, OMR_GET_CALLSITE(), NULL);

	static void
	eventOccurred(J9HookInterface** hookInterface, UDATA eventNum, void* voidData, void* userData)
	{
		J9VMPutStaticFieldEvent* eventData = voidData;
		. . .
	}
 */
#define J9HOOK_VM_PUT_STATIC_FIELD 49
typedef struct J9VMPutStaticFieldEvent {
	struct J9VMThread* currentThread;
	struct J9Method* method;
	IDATA location;
	struct J9Class* declaringClass;
	void* fieldAddress;
	U_64 newValue;
} J9VMPutStaticFieldEvent;

/* J9HOOK_VM_SINGLE_STEP

			Triggered before every bytecode execution within a method.
			This event is not triggered by native methods, and JIT methods will be decompiled in order to report this event.
			Note that hooking this event incurs an astronomical performance penalty.
		

Example usage:
	(*hookable)->J9HookRegisterWithCallSite(hookable, J9HOOK_VM_SINGLE_STEP, eventOccurred, OMR_GET_CALLSITE(), NULL);

	static void
	eventOccurred(J9HookInterface** hookInterface, UDATA eventNum, void* voidData, void* userData)
	{
		J9VMSingleStepEvent* eventData = voidData;
		. . .
	}
 */
#define J9HOOK_VM_SINGLE_STEP 50
typedef struct J9VMSingleStepEvent {
	struct J9VMThread* currentThread;
	struct J9Method* method;
	IDATA location;
} J9VMSingleStepEvent;

/* J9HOOK_VM_PROFILING_BYTECODE_BUFFER_FULL

			If this event is hooked, the VM installs profiling versions of branch and call
			bytecodes and collects profiling data about these bytecodes in buffers associated
			with each thread. When a buffer becomes full, this event is triggered on the
			thread which filled its buffer. Once listeners have been notified, the buffer is reset
			any pointers into the buffer become invalid.

			When classes are unloaded, or when a thread is terminated, any unprocessed
			data remaining in a thread's buffer is discarded without triggering this event.

			If this event is unhooked, the profiling buffers remain in place, and profiling will
			resume where it left off if the event is re-hooked.

			Listeners should not release VM access during this event, as this may allow additional
			bytecodes to run.

			Records in the buffer all start with a (possibly misaligned) pointer to the bytecode
			which was profiled. Listeners must inspect the bytecode to determine what follows:
				lookupswitch:
					4 bytes: the operand (i.e. the key) to the lookupswitch
				tableswitch:
					4 bytes: the operand (i.e. the index) to the tableswitch
				ifacmpeq,ifacmpne,ifeq,ifge,ifgt,ifle,iflt,ifne,ificmpeq,ificmpne,ificmpge,ificmpgt,ificmple,ificmplt,ifnull,ifnonnull:
					1 byte: 1 for 'branch taken', 0 for 'branch not taken'
				invokevirtual,invokeinterface,invokeinterface2:
					4 or 8 bytes: a pointer to the class of the receiver. No record is generated if the receiver is null.
				checkcast,instanceof:
					4 or 8 bytes: a pointer to the class of the instance being cast. No record is generated if the instance is null.

			Since records point directly at bytecodes, this event disables itself silently while breakpointed methods
			are in use, as records may point into breakpointed methods which no longer exist. The event may still be
			hooked, but will not be triggered.

			This event should be regarded as a 'best-effort' contract with the interpreter. The interpreter
			will attempt to report all data, but it is permitted to discard data, or stop collecting with no
			notice.
		

Example usage:
	(*hookable)->J9HookRegisterWithCallSite(hookable, J9HOOK_VM_PROFILING_BYTECODE_BUFFER_FULL, eventOccurred, OMR_GET_CALLSITE(), NULL);

	static void
	eventOccurred(J9HookInterface** hookInterface, UDATA eventNum, void* voidData, void* userData)
	{
		J9VMProfilingBytecodeBufferFullEvent* eventData = voidData;
		. . .
	}
 */
#if defined (J9VM_INTERP_PROFILING_BYTECODES)
#define J9HOOK_VM_PROFILING_BYTECODE_BUFFER_FULL 51
typedef struct J9VMProfilingBytecodeBufferFullEvent {
	struct J9VMThread* currentThread;
	const U_8* bufferStart;
	UDATA bufferSize;
} J9VMProfilingBytecodeBufferFullEvent;
#endif /* defined (J9VM_INTERP_PROFILING_BYTECODES)*/

/* J9HOOK_VM_CLASS_PREINITIALIZE

			Triggered just before a class is initialized.
			The current thread has VM access.
		

Example usage:
	(*hookable)->J9HookRegisterWithCallSite(hookable, J9HOOK_VM_CLASS_PREINITIALIZE, eventOccurred, OMR_GET_CALLSITE(), NULL);

	static void
	eventOccurred(J9HookInterface** hookInterface, UDATA eventNum, void* voidData, void* userData)
	{
		J9VMClassPreinitializeEvent* eventData = voidData;
		. . .
	}
 */
#define J9HOOK_VM_CLASS_PREINITIALIZE 52
typedef struct J9VMClassPreinitializeEvent {
	struct J9VMThread* currentThread;
	struct J9Class* clazz;
	UDATA failed;
} J9VMClassPreinitializeEvent;

/* J9HOOK_VM_CLASS_INITIALIZE

			Triggered just after a class is succesfully initialized.
			The current thread has VM access.
		

Example usage:
	(*hookable)->J9HookRegisterWithCallSite(hookable, J9HOOK_VM_CLASS_INITIALIZE, eventOccurred, OMR_GET_CALLSITE(), NULL);

	static void
	eventOccurred(J9HookInterface** hookInterface, UDATA eventNum, void* voidData, void* userData)
	{
		J9VMClassInitializeEvent* eventData = voidData;
		. . .
	}
 */
#define J9HOOK_VM_CLASS_INITIALIZE 53
typedef struct J9VMClassInitializeEvent {
	struct J9VMThread* currentThread;
	struct J9Class* clazz;
} J9VMClassInitializeEvent;

/* J9HOOK_VM_CLASS_INITIALIZE_FAILED

			Triggered just after a class is succesfully initialized.
			The current thread has VM access, and an exception is pending.
		

Example usage:
	(*hookable)->J9HookRegisterWithCallSite(hookable, J9HOOK_VM_CLASS_INITIALIZE_FAILED, eventOccurred, OMR_GET_CALLSITE(), NULL);

	static void
	eventOccurred(J9HookInterface** hookInterface, UDATA eventNum, void* voidData, void* userData)
	{
		J9VMClassInitializeFailedEvent* eventData = voidData;
		. . .
	}
 */
#define J9HOOK_VM_CLASS_INITIALIZE_FAILED 54
typedef struct J9VMClassInitializeFailedEvent {
	struct J9VMThread* currentThread;
	struct J9Class* clazz;
} J9VMClassInitializeFailedEvent;

/* J9HOOK_VM_CLASS_LOADER_INITIALIZED

			Triggered just after a classloader is succesfully initialized.
			The current thread has VM access.
		

Example usage:
	(*hookable)->J9HookRegisterWithCallSite(hookable, J9HOOK_VM_CLASS_LOADER_INITIALIZED, eventOccurred, OMR_GET_CALLSITE(), NULL);

	static void
	eventOccurred(J9HookInterface** hookInterface, UDATA eventNum, void* voidData, void* userData)
	{
		J9VMClassLoaderInitializedEvent* eventData = voidData;
		. . .
	}
 */
#define J9HOOK_VM_CLASS_LOADER_INITIALIZED 55
typedef struct J9VMClassLoaderInitializedEvent {
	struct J9VMThread* vmThread;
	struct J9ClassLoader* classLoader;
} J9VMClassLoaderInitializedEvent;

/* J9HOOK_VM_LOOKUP_JNI_ID

			Triggered whenever a GetFieldID, GetStaticFieldID, GetMethodID, or GetStaticMethodID call is about to return
		

Example usage:
	(*hookable)->J9HookRegisterWithCallSite(hookable, J9HOOK_VM_LOOKUP_JNI_ID, eventOccurred, OMR_GET_CALLSITE(), NULL);

	static void
	eventOccurred(J9HookInterface** hookInterface, UDATA eventNum, void* voidData, void* userData)
	{
		J9VMLookupJNIIDEvent* eventData = voidData;
		. . .
	}
 */
#define J9HOOK_VM_LOOKUP_JNI_ID 56
typedef struct J9VMLookupJNIIDEvent {
	struct J9VMThread* currentThread;
	jclass clazz;
	const char * name;
	const char * signature;
	U_8 isStatic;
	U_8 isField;
	void * id;
} J9VMLookupJNIIDEvent;

/* J9HOOK_VM_FIND_LOCALLY_DEFINED_CLASS

			Triggered when a classloader is looking for a locally defined class. It looks first in its local cache,
			then this hook allows a ROMClass to be provided before the classloader looks on disk.
			Use the class path entries from classloader->classPathEntries if it is bootstrap class loader. Otherwise, use
			classPathEntries passed in.
		

Example usage:
	(*hookable)->J9HookRegisterWithCallSite(hookable, J9HOOK_VM_FIND_LOCALLY_DEFINED_CLASS, eventOccurred, OMR_GET_CALLSITE(), NULL);

	static void
	eventOccurred(J9HookInterface** hookInterface, UDATA eventNum, void* voidData, void* userData)
	{
		J9VMFindLocalClassEvent* eventData = voidData;
		. . .
	}
 */
#define J9HOOK_VM_FIND_LOCALLY_DEFINED_CLASS 57
typedef struct J9VMFindLocalClassEvent {
	struct J9VMThread* currentThread;
	struct J9ClassLoader* classloader;
	struct J9Module* j9module;
	const char* className;
	UDATA classNameLength;
	struct J9ClassPathEntry** classPathEntries;
	UDATA entryCount;
	IDATA confirmedCount;
	const struct J9UTF8* partition;
	UDATA doPreventFind;
	UDATA doPreventStore;
	IDATA* foundAtIndex;
	struct J9ROMClass* result;
} J9VMFindLocalClassEvent;

/* J9HOOK_VM_FRAME_POP

			Triggered every time a method is about to return. Hooking this event may have a significant performance impact.
			Even reserving this event will cause the VM and JIT to run with reduced optimization.
			When this event is reported, the current thread has VM access.
			This event is reported after J9HOOK_VM_METHOD_RETURN if both are hooked.
		

Example usage:
	(*hookable)->J9HookRegisterWithCallSite(hookable, J9HOOK_VM_FRAME_POP, eventOccurred, OMR_GET_CALLSITE(), NULL);

	static void
	eventOccurred(J9HookInterface** hookInterface, UDATA eventNum, void* voidData, void* userData)
	{
		J9VMFramePopEvent* eventData = voidData;
		. . .
	}
 */
#define J9HOOK_VM_FRAME_POP 58
typedef struct J9VMFramePopEvent {
	struct J9VMThread* currentThread;
	struct J9Method* method;
	UDATA poppedByException;
} J9VMFramePopEvent;

/* J9HOOK_VM_INTERNAL_CLASS_LOAD

				Triggered immediately after a RAM class has been created for a newly loaded class.
				Static final fields have been initialized to their default values (zero).
				VM access and the class table mutex are both owned by the current thread, so the class
				is not yet visible to other threads. It is NOT safe to execute Java code from handlers of this
				hook.
		

Example usage:
	(*hookable)->J9HookRegisterWithCallSite(hookable, J9HOOK_VM_INTERNAL_CLASS_LOAD, eventOccurred, OMR_GET_CALLSITE(), NULL);

	static void
	eventOccurred(J9HookInterface** hookInterface, UDATA eventNum, void* voidData, void* userData)
	{
		J9VMInternalClassLoadEvent* eventData = voidData;
		. . .
	}
 */
#define J9HOOK_VM_INTERNAL_CLASS_LOAD 59
typedef struct J9VMInternalClassLoadEvent {
	struct J9VMThread* currentThread;
	struct J9Class* clazz;
	UDATA failed;
} J9VMInternalClassLoadEvent;

/* J9HOOK_VM_POP_FRAMES_INTERRUPT

				Triggered when control is returned to a java method and publicFlagsPopFramesInterrupt
				is set in the current thread's publicFlags.  Set the "executeAsyncCheck" field to FALSE to execute
				the current bytecode upon return, or TRUE to execute another async check.
		

Example usage:
	(*hookable)->J9HookRegisterWithCallSite(hookable, J9HOOK_VM_POP_FRAMES_INTERRUPT, eventOccurred, OMR_GET_CALLSITE(), NULL);

	static void
	eventOccurred(J9HookInterface** hookInterface, UDATA eventNum, void* voidData, void* userData)
	{
		J9VMPopFramesInterruptEvent* eventData = voidData;
		. . .
	}
 */
#if defined (J9VM_INTERP_HOT_CODE_REPLACEMENT)
#define J9HOOK_VM_POP_FRAMES_INTERRUPT 60
typedef struct J9VMPopFramesInterruptEvent {
	struct J9VMThread* currentThread;
	UDATA executeAsyncCheck;
} J9VMPopFramesInterruptEvent;
#endif /* defined (J9VM_INTERP_HOT_CODE_REPLACEMENT)*/

/* J9HOOK_VM_LOOKUP_NATIVE_ADDRESS

				Triggered when a native cannot be bound in using the ClassLoader's native libraries.
				VM access is not held during this event.
		

Example usage:
	(*hookable)->J9HookRegisterWithCallSite(hookable, J9HOOK_VM_LOOKUP_NATIVE_ADDRESS, eventOccurred, OMR_GET_CALLSITE(), NULL);

	static void
	eventOccurred(J9HookInterface** hookInterface, UDATA eventNum, void* voidData, void* userData)
	{
		J9VMLookupNativeAddressEvent* eventData = voidData;
		. . .
	}
 */
#if defined (J9VM_OPT_JVMTI)
#define J9HOOK_VM_LOOKUP_NATIVE_ADDRESS 61
typedef struct J9VMLookupNativeAddressEvent {
	struct J9VMThread * currentThread;
	struct J9Method * nativeMethod;
	char * longJNI;
	char * shortJNI;
	UDATA functionArgCount;
	lookupNativeAddressCallback callback;
} J9VMLookupNativeAddressEvent;
#endif /* defined (J9VM_OPT_JVMTI)*/

/* J9HOOK_VM_FIND_NATIVE_TO_REGISTER

				Triggered when RegisterNatives finds a matching method which is not native.
				This hook has the opportunity to find a different method to which to bind.
		

Example usage:
	(*hookable)->J9HookRegisterWithCallSite(hookable, J9HOOK_VM_FIND_NATIVE_TO_REGISTER, eventOccurred, OMR_GET_CALLSITE(), NULL);

	static void
	eventOccurred(J9HookInterface** hookInterface, UDATA eventNum, void* voidData, void* userData)
	{
		J9VMFindNativeToRegisterEvent* eventData = voidData;
		. . .
	}
 */
#if defined (J9VM_OPT_JVMTI)
#define J9HOOK_VM_FIND_NATIVE_TO_REGISTER 62
typedef struct J9VMFindNativeToRegisterEvent {
	struct J9VMThread* currentThread;
	struct J9Method* method;
} J9VMFindNativeToRegisterEvent;
#endif /* defined (J9VM_OPT_JVMTI)*/

/* J9HOOK_VM_JNI_NATIVE_BIND

				Triggered when a JNI native is about to be bound.
				nativeMethodAddress may be modified by the listener.
		

Example usage:
	(*hookable)->J9HookRegisterWithCallSite(hookable, J9HOOK_VM_JNI_NATIVE_BIND, eventOccurred, OMR_GET_CALLSITE(), NULL);

	static void
	eventOccurred(J9HookInterface** hookInterface, UDATA eventNum, void* voidData, void* userData)
	{
		J9VMJNINativeBindEvent* eventData = voidData;
		. . .
	}
 */
#if defined (J9VM_OPT_JVMTI)
#define J9HOOK_VM_JNI_NATIVE_BIND 63
typedef struct J9VMJNINativeBindEvent {
	struct J9VMThread* currentThread;
	struct J9Method * nativeMethod;
	void * nativeMethodAddress;
} J9VMJNINativeBindEvent;
#endif /* defined (J9VM_OPT_JVMTI)*/

/* J9HOOK_VM_JNI_NATIVE_REGISTERED

				Triggered when a JNI native is about to be bound (or rebound) by JNI RegisterNatives.
				The current thread has exclusive VM access.  This hook is called before the new
				address is written into the method.
		

Example usage:
	(*hookable)->J9HookRegisterWithCallSite(hookable, J9HOOK_VM_JNI_NATIVE_REGISTERED, eventOccurred, OMR_GET_CALLSITE(), NULL);

	static void
	eventOccurred(J9HookInterface** hookInterface, UDATA eventNum, void* voidData, void* userData)
	{
		J9VMJNINativeRegisteredEvent* eventData = voidData;
		. . .
	}
 */
#define J9HOOK_VM_JNI_NATIVE_REGISTERED 64
typedef struct J9VMJNINativeRegisteredEvent {
	struct J9VMThread* currentThread;
	struct J9Method * nativeMethod;
	void * nativeMethodAddress;
} J9VMJNINativeRegisteredEvent;

/* J9HOOK_VM_INITIALIZE_REQUIRED_CLASSES_DONE

				Triggered when initializeRequiredClasses has completed.  Hook will be invoked
				_without_ VM access.
		

Example usage:
	(*hookable)->J9HookRegisterWithCallSite(hookable, J9HOOK_VM_INITIALIZE_REQUIRED_CLASSES_DONE, eventOccurred, OMR_GET_CALLSITE(), NULL);

	static void
	eventOccurred(J9HookInterface** hookInterface, UDATA eventNum, void* voidData, void* userData)
	{
		J9VMInitializeRequiredClassesDoneEvent* eventData = voidData;
		. . .
	}
 */
#define J9HOOK_VM_INITIALIZE_REQUIRED_CLASSES_DONE 65
typedef struct J9VMInitializeRequiredClassesDoneEvent {
	struct J9VMThread* currentThread;
	IDATA continueInitialization;
} J9VMInitializeRequiredClassesDoneEvent;

/* J9HOOK_VM_INITIALIZE_TENANTDATA_DONE

				Triggered when the initialization to tenant data has completed.
				VMAccess will be held when this hook is called
		

Example usage:
	(*hookable)->J9HookRegisterWithCallSite(hookable, J9HOOK_VM_INITIALIZE_TENANTDATA_DONE, eventOccurred, OMR_GET_CALLSITE(), NULL);

	static void
	eventOccurred(J9HookInterface** hookInterface, UDATA eventNum, void* voidData, void* userData)
	{
		J9VMInitializeTenantDataDoneEvent* eventData = voidData;
		. . .
	}
 */
#define J9HOOK_VM_INITIALIZE_TENANTDATA_DONE 66
typedef struct J9VMInitializeTenantDataDoneEvent {
	struct J9VMThread* currentThread;
} J9VMInitializeTenantDataDoneEvent;

/* J9HOOK_VM_DYNAMIC_CODE_LOAD

			Triggered when dynamic code is generated (e.g. JIT compiles a method). Multiple events may be generated for
			the same method.  If method is NULL, name must be non-NULL.  If method is non-NULL, then name is optional.
			The name string does not persist beyond the reporting of this event - it must be copied by the listener if desired.

			This hook may be called with or without VM access, may be called with exclusive VM access, and may be called
			while internal VM mutexxes are held.
		

Example usage:
	(*hookable)->J9HookRegisterWithCallSite(hookable, J9HOOK_VM_DYNAMIC_CODE_LOAD, eventOccurred, OMR_GET_CALLSITE(), NULL);

	static void
	eventOccurred(J9HookInterface** hookInterface, UDATA eventNum, void* voidData, void* userData)
	{
		J9DynamicCodeLoadEvent* eventData = voidData;
		. . .
	}
 */
#if defined (J9VM_INTERP_NATIVE_SUPPORT)
#define J9HOOK_VM_DYNAMIC_CODE_LOAD 67
typedef struct J9DynamicCodeLoadEvent {
	struct J9VMThread* currentThread;
	struct J9Method* method;
	void * startPC;
	UDATA length;
	const char * name;
	void * metaData;
} J9DynamicCodeLoadEvent;
#endif /* defined (J9VM_INTERP_NATIVE_SUPPORT)*/

/* J9HOOK_VM_DYNAMIC_CODE_UNLOAD

			Triggered when dynamic code is unloaded.

			This hook may be called with or without VM access, may be called with exclusive VM access, and may be called
			while internal VM mutexxes are held.
		

Example usage:
	(*hookable)->J9HookRegisterWithCallSite(hookable, J9HOOK_VM_DYNAMIC_CODE_UNLOAD, eventOccurred, OMR_GET_CALLSITE(), NULL);

	static void
	eventOccurred(J9HookInterface** hookInterface, UDATA eventNum, void* voidData, void* userData)
	{
		J9DynamicCodeUnloadEvent* eventData = voidData;
		. . .
	}
 */
#if defined (J9VM_INTERP_NATIVE_SUPPORT)
#define J9HOOK_VM_DYNAMIC_CODE_UNLOAD 68
typedef struct J9DynamicCodeUnloadEvent {
	struct J9VMThread* currentThread;
	struct J9Method* method;
	void * startPC;
} J9DynamicCodeUnloadEvent;
#endif /* defined (J9VM_INTERP_NATIVE_SUPPORT)*/

/* J9HOOK_VM_CLASS_LOADER_UNLOAD

			Triggered when a classloader is about to be unloaded.
			This event will be triggered for each loader which is to be unloaded
			in the current GC cycle before any of them, or their classes, are actually freed.
			J9HOOK_VM_CLASSES_UNLOAD will have been triggered before this event.
		

Example usage:
	(*hookable)->J9HookRegisterWithCallSite(hookable, J9HOOK_VM_CLASS_LOADER_UNLOAD, eventOccurred, OMR_GET_CALLSITE(), NULL);

	static void
	eventOccurred(J9HookInterface** hookInterface, UDATA eventNum, void* voidData, void* userData)
	{
		J9VMClassLoaderUnloadEvent* eventData = voidData;
		. . .
	}
 */
#if defined (J9VM_GC_DYNAMIC_CLASS_UNLOADING)
#define J9HOOK_VM_CLASS_LOADER_UNLOAD 69
typedef struct J9VMClassLoaderUnloadEvent {
	struct J9VMThread* currentThread;
	struct J9ClassLoader* classLoader;
} J9VMClassLoaderUnloadEvent;
#endif /* defined (J9VM_GC_DYNAMIC_CLASS_UNLOADING)*/

/* J9HOOK_VM_CLASS_LOADERS_UNLOAD

			Triggered when a set of class loaders is about to be unloaded.
		

Example usage:
	(*hookable)->J9HookRegisterWithCallSite(hookable, J9HOOK_VM_CLASS_LOADERS_UNLOAD, eventOccurred, OMR_GET_CALLSITE(), NULL);

	static void
	eventOccurred(J9HookInterface** hookInterface, UDATA eventNum, void* voidData, void* userData)
	{
		J9VMClassLoadersUnloadEvent* eventData = voidData;
		. . .
	}
 */
#if defined (J9VM_GC_DYNAMIC_CLASS_UNLOADING)
#define J9HOOK_VM_CLASS_LOADERS_UNLOAD 70
typedef struct J9VMClassLoadersUnloadEvent {
	struct J9VMThread* currentThread;
	struct J9ClassLoader* classLoadersToUnload;
} J9VMClassLoadersUnloadEvent;
#endif /* defined (J9VM_GC_DYNAMIC_CLASS_UNLOADING)*/

/* J9HOOK_VM_CLASS_UNLOAD

			Triggered when a class is about to be unloaded.
			This event will be triggered for each class which is to be unloaded
			in the current GC cycle before any of them are actually freed.
			J9HOOK_VM_CLASSES_UNLOAD will be triggered after this event has
			been triggered for all of the unloading classes.
		

Example usage:
	(*hookable)->J9HookRegisterWithCallSite(hookable, J9HOOK_VM_CLASS_UNLOAD, eventOccurred, OMR_GET_CALLSITE(), NULL);

	static void
	eventOccurred(J9HookInterface** hookInterface, UDATA eventNum, void* voidData, void* userData)
	{
		J9VMClassUnloadEvent* eventData = voidData;
		. . .
	}
 */
#if defined (J9VM_GC_DYNAMIC_CLASS_UNLOADING)
#define J9HOOK_VM_CLASS_UNLOAD 71
typedef struct J9VMClassUnloadEvent {
	struct J9VMThread* currentThread;
	struct J9Class* clazz;
} J9VMClassUnloadEvent;
#endif /* defined (J9VM_GC_DYNAMIC_CLASS_UNLOADING)*/

/* J9HOOK_VM_CLASSES_UNLOAD

			Triggered when a set of classes is about to be unloaded.
			This event will be triggered once before any of the classes including anonymous are unloaded
			in the current GC cycle, after J9HOOK_VM_CLASS_UNLOAD has been
			triggered for each of the unloading classes.  Partial clean up may have been
			performed for the unloading classes prior to this event.
		

Example usage:
	(*hookable)->J9HookRegisterWithCallSite(hookable, J9HOOK_VM_CLASSES_UNLOAD, eventOccurred, OMR_GET_CALLSITE(), NULL);

	static void
	eventOccurred(J9HookInterface** hookInterface, UDATA eventNum, void* voidData, void* userData)
	{
		J9VMClassesUnloadEvent* eventData = voidData;
		. . .
	}
 */
#if defined (J9VM_GC_DYNAMIC_CLASS_UNLOADING)
#define J9HOOK_VM_CLASSES_UNLOAD 72
typedef struct J9VMClassesUnloadEvent {
	struct J9VMThread* currentThread;
	UDATA classUnloadCount;
	struct J9Class* classesToUnload;
} J9VMClassesUnloadEvent;
#endif /* defined (J9VM_GC_DYNAMIC_CLASS_UNLOADING)*/

/* J9HOOK_VM_ANON_CLASSES_UNLOAD

			Triggered when a set of anonymous classes is about to be unloaded.
			This event will be triggered once before any of the anonymous classes are unloaded
			in the current GC cycle, after J9HOOK_VM_CLASS_UNLOAD has been
			triggered for each of the unloading classes.  Partial clean up may have been
			performed for the unloading classes prior to this event.
		

Example usage:
	(*hookable)->J9HookRegisterWithCallSite(hookable, J9HOOK_VM_ANON_CLASSES_UNLOAD, eventOccurred, OMR_GET_CALLSITE(), NULL);

	static void
	eventOccurred(J9HookInterface** hookInterface, UDATA eventNum, void* voidData, void* userData)
	{
		J9VMAnonymousClassesUnloadEvent* eventData = voidData;
		. . .
	}
 */
#if defined (J9VM_GC_DYNAMIC_CLASS_UNLOADING)
#define J9HOOK_VM_ANON_CLASSES_UNLOAD 73
typedef struct J9VMAnonymousClassesUnloadEvent {
	struct J9VMThread* currentThread;
	UDATA anonymousClassUnloadCount;
	struct J9Class* anonymousClassesToUnload;
} J9VMAnonymousClassesUnloadEvent;
#endif /* defined (J9VM_GC_DYNAMIC_CLASS_UNLOADING)*/

/* J9HOOK_VM_STORE_COMPILED_METHOD

			Triggered when a method has been compiled.
		

Example usage:
	(*hookable)->J9HookRegisterWithCallSite(hookable, J9HOOK_VM_STORE_COMPILED_METHOD, eventOccurred, OMR_GET_CALLSITE(), NULL);

	static void
	eventOccurred(J9HookInterface** hookInterface, UDATA eventNum, void* voidData, void* userData)
	{
		J9VMStoreCompiledMethodEvent* eventData = voidData;
		. . .
	}
 */
#define J9HOOK_VM_STORE_COMPILED_METHOD 74
typedef struct J9VMStoreCompiledMethodEvent {
	struct J9VMThread* currentThread;
	struct J9ROMMethod* romMethod;
	const U_8* dataStart;
	UDATA dataSize;
	const U_8* codeStart;
	UDATA codeSize;
	const U_8* result;
} J9VMStoreCompiledMethodEvent;

/* J9HOOK_VM_FIND_COMPILED_METHOD

			Triggered when a compiled method is retrieved.
		

Example usage:
	(*hookable)->J9HookRegisterWithCallSite(hookable, J9HOOK_VM_FIND_COMPILED_METHOD, eventOccurred, OMR_GET_CALLSITE(), NULL);

	static void
	eventOccurred(J9HookInterface** hookInterface, UDATA eventNum, void* voidData, void* userData)
	{
		J9VMFindCompiledMethodEvent* eventData = voidData;
		. . .
	}
 */
#define J9HOOK_VM_FIND_COMPILED_METHOD 75
typedef struct J9VMFindCompiledMethodEvent {
	struct J9VMThread* currentThread;
	struct J9ROMMethod* romMethod;
	const U_8* result;
} J9VMFindCompiledMethodEvent;

/* J9HOOK_VM_DUMP_START
Triggered whenever a dump starts

Example usage:
	(*hookable)->J9HookRegisterWithCallSite(hookable, J9HOOK_VM_DUMP_START, eventOccurred, OMR_GET_CALLSITE(), NULL);

	static void
	eventOccurred(J9HookInterface** hookInterface, UDATA eventNum, void* voidData, void* userData)
	{
		J9VMVmDumpStartEvent* eventData = voidData;
		. . .
	}
 */
#define J9HOOK_VM_DUMP_START 76
typedef struct J9VMVmDumpStartEvent {
	struct J9VMThread* currentThread;
	char * label;
	char * detail;
} J9VMVmDumpStartEvent;

/* J9HOOK_VM_DUMP_END
Triggered whenever a dump ends

Example usage:
	(*hookable)->J9HookRegisterWithCallSite(hookable, J9HOOK_VM_DUMP_END, eventOccurred, OMR_GET_CALLSITE(), NULL);

	static void
	eventOccurred(J9HookInterface** hookInterface, UDATA eventNum, void* voidData, void* userData)
	{
		J9VMVmDumpEndEvent* eventData = voidData;
		. . .
	}
 */
#define J9HOOK_VM_DUMP_END 77
typedef struct J9VMVmDumpEndEvent {
	struct J9VMThread* currentThread;
	char * label;
	char * detail;
} J9VMVmDumpEndEvent;

/* J9HOOK_VM_RESOURCE_EXHAUSTED
Triggered whenever a VM resource such as memory or threads is exhausted

Example usage:
	(*hookable)->J9HookRegisterWithCallSite(hookable, J9HOOK_VM_RESOURCE_EXHAUSTED, eventOccurred, OMR_GET_CALLSITE(), NULL);

	static void
	eventOccurred(J9HookInterface** hookInterface, UDATA eventNum, void* voidData, void* userData)
	{
		J9VMVmResourceExhaustedEvent* eventData = voidData;
		. . .
	}
 */
#define J9HOOK_VM_RESOURCE_EXHAUSTED 78
typedef struct J9VMVmResourceExhaustedEvent {
	struct J9VMThread* currentThread;
	UDATA resourceTypes;
	const char * description;
} J9VMVmResourceExhaustedEvent;

/* J9HOOK_VM_LOAD_JNI_LIBRARY

			Triggered during symbolic resolution of JNI library (e.g. System.loadLibrary('zz')) this
			hook allows alternative resolution of the name (the 'zz' above).  The nativeLibrary passed
			to the hook will have default resolution functions, which may be overridden in the hook.

			Callee must fill in the result field to indicate what action was taken:
				J9NATIVE_LIBRARY_LOAD_OK	 	 	= The library was loaded successfully.
				J9NATIVE_LIBRARY_LOAD_NOT_FOUND  	= The library was not found (non-fatal).
				J9NATIVE_LIBRARY_LOAD_ERROR 	 	= The library failed to load (fatal), errorBuf has details.
				J9NATIVE_LIBRARY_LOAD_INIT_ERROR 	= JNI_OnLoad() failed (fatal), errorBuf has details.
				J9NATIVE_LIBRARY_LOAD_GENERIC_ERROR = A generic error occurred, errorBuf has details.
		

Example usage:
	(*hookable)->J9HookRegisterWithCallSite(hookable, J9HOOK_VM_LOAD_JNI_LIBRARY, eventOccurred, OMR_GET_CALLSITE(), NULL);

	static void
	eventOccurred(J9HookInterface** hookInterface, UDATA eventNum, void* voidData, void* userData)
	{
		J9VMLoadJNILibraryEvent* eventData = voidData;
		. . .
	}
 */
#define J9HOOK_VM_LOAD_JNI_LIBRARY 79
typedef struct J9VMLoadJNILibraryEvent {
	struct J9VMThread* currentThread;
	struct J9ClassLoader*  classLoader;
	char * logicalName;
	struct J9NativeLibrary * nativeLibrary;
	UDATA result;
	char * errorBuf;
	UDATA bufLen;
	UDATA openFlags;
} J9VMLoadJNILibraryEvent;

/* J9HOOK_VM_SERIALIZE_SHARED_CACHE
Trigger to request the shared cache be serialized.

Example usage:
	(*hookable)->J9HookRegisterWithCallSite(hookable, J9HOOK_VM_SERIALIZE_SHARED_CACHE, eventOccurred, OMR_GET_CALLSITE(), NULL);

	static void
	eventOccurred(J9HookInterface** hookInterface, UDATA eventNum, void* voidData, void* userData)
	{
		J9VMSerializeSharedCacheEvent* eventData = voidData;
		. . .
	}
 */
#define J9HOOK_VM_SERIALIZE_SHARED_CACHE 80
typedef struct J9VMSerializeSharedCacheEvent {
	struct J9JavaVM* vm;
	IDATA result;
} J9VMSerializeSharedCacheEvent;

/* J9HOOK_VM_CLASS_LOADER_CREATED
This event is triggered when a classloader is created

Example usage:
	(*hookable)->J9HookRegisterWithCallSite(hookable, J9HOOK_VM_CLASS_LOADER_CREATED, eventOccurred, OMR_GET_CALLSITE(), NULL);

	static void
	eventOccurred(J9HookInterface** hookInterface, UDATA eventNum, void* voidData, void* userData)
	{
		J9VMClassloaderCreatedEvent* eventData = voidData;
		. . .
	}
 */
#define J9HOOK_VM_CLASS_LOADER_CREATED 81
typedef struct J9VMClassloaderCreatedEvent {
	struct J9JavaVM* vm;
	struct J9ClassLoader* classLoader;
} J9VMClassloaderCreatedEvent;

/* J9HOOK_VM_CLASS_LOADER_DESTROY
This event is triggered when a classloader is destroyed

Example usage:
	(*hookable)->J9HookRegisterWithCallSite(hookable, J9HOOK_VM_CLASS_LOADER_DESTROY, eventOccurred, OMR_GET_CALLSITE(), NULL);

	static void
	eventOccurred(J9HookInterface** hookInterface, UDATA eventNum, void* voidData, void* userData)
	{
		J9VMClassloaderDestroyEvent* eventData = voidData;
		. . .
	}
 */
#define J9HOOK_VM_CLASS_LOADER_DESTROY 82
typedef struct J9VMClassloaderDestroyEvent {
	struct J9JavaVM* vm;
	struct J9ClassLoader* classLoader;
} J9VMClassloaderDestroyEvent;

/* J9HOOK_VM_CLASS_LOADER_CLASSPATH_ENTRY_ADDED
This event is triggered when a class path entry is added to a ClassLoader for which
				     the classpath was already initialized 

Example usage:
	(*hookable)->J9HookRegisterWithCallSite(hookable, J9HOOK_VM_CLASS_LOADER_CLASSPATH_ENTRY_ADDED, eventOccurred, OMR_GET_CALLSITE(), NULL);

	static void
	eventOccurred(J9HookInterface** hookInterface, UDATA eventNum, void* voidData, void* userData)
	{
		J9VMClassloaderClassPathEntryAddedEvent* eventData = voidData;
		. . .
	}
 */
#define J9HOOK_VM_CLASS_LOADER_CLASSPATH_ENTRY_ADDED 83
typedef struct J9VMClassloaderClassPathEntryAddedEvent {
	struct J9JavaVM* vm;
	struct J9ClassLoader* classLoader;
	struct J9ClassPathEntry* classPathEntry;
} J9VMClassloaderClassPathEntryAddedEvent;

/* J9HOOK_VM_CLASS_LOADER_CLASSPATH_ENTRIES_INITIALIZED
This event is triggered when a the initial classpath for a classloader is ready 

Example usage:
	(*hookable)->J9HookRegisterWithCallSite(hookable, J9HOOK_VM_CLASS_LOADER_CLASSPATH_ENTRIES_INITIALIZED, eventOccurred, OMR_GET_CALLSITE(), NULL);

	static void
	eventOccurred(J9HookInterface** hookInterface, UDATA eventNum, void* voidData, void* userData)
	{
		J9VMClassloaderClassPathEntriesInitializedEvent* eventData = voidData;
		. . .
	}
 */
#define J9HOOK_VM_CLASS_LOADER_CLASSPATH_ENTRIES_INITIALIZED 84
typedef struct J9VMClassloaderClassPathEntriesInitializedEvent {
	struct J9JavaVM* vm;
	struct J9ClassLoader* classLoader;
} J9VMClassloaderClassPathEntriesInitializedEvent;

/* J9HOOK_VM_FIND_LOCALLY_DEFINED_CLASS_FROM_FILESYSTEM

			Triggered when a classloader is looking for a locally defined class using its classpath.  This occurs after
			lookups to the shared cache or other methods that do not use the files specified by the classpath.  If the
			hook method is run it should update searchComplete to 1 regardless of wether or not the class was found.  In this
			case the vm will not search for the class itself.
		

Example usage:
	(*hookable)->J9HookRegisterWithCallSite(hookable, J9HOOK_VM_FIND_LOCALLY_DEFINED_CLASS_FROM_FILESYSTEM, eventOccurred, OMR_GET_CALLSITE(), NULL);

	static void
	eventOccurred(J9HookInterface** hookInterface, UDATA eventNum, void* voidData, void* userData)
	{
		J9VMFindLocalClassFromFilesystemEvent* eventData = voidData;
		. . .
	}
 */
#define J9HOOK_VM_FIND_LOCALLY_DEFINED_CLASS_FROM_FILESYSTEM 85
typedef struct J9VMFindLocalClassFromFilesystemEvent {
	struct J9VMThread* currentThread;
	const char* className;
	UDATA classNameLength;
	struct J9ClassLoader* classloader;
	IDATA* classFound;
	IDATA* result;
} J9VMFindLocalClassFromFilesystemEvent;

/* J9HOOK_VM_SCAR_PRECONFIGURE

			Triggered when we are about to preconfigure for the sidecar class library.  If returnValue
			is still null on return then the normal preconfigure is completed
		

Example usage:
	(*hookable)->J9HookRegisterWithCallSite(hookable, J9HOOK_VM_SCAR_PRECONFIGURE, eventOccurred, OMR_GET_CALLSITE(), NULL);

	static void
	eventOccurred(J9HookInterface** hookInterface, UDATA eventNum, void* voidData, void* userData)
	{
		J9VMScarPreconfigureEvent* eventData = voidData;
		. . .
	}
 */
#define J9HOOK_VM_SCAR_PRECONFIGURE 86
typedef struct J9VMScarPreconfigureEvent {
	struct J9JavaVM* vm;
	I_32* returnValue;
	IDATA* result;
} J9VMScarPreconfigureEvent;

/* J9HOOK_VM_OBJECT_ALLOCATE_WITHIN_THRESHOLD

			Triggered whenever an object is allocated whose size is within the range provided by lowThreshold and highThreshold (inclusive).
			The current thread has VM access, but a handler may safely release and reacquire access.  If VM access is released,
			the listener is responsible for keeping the object up-to-date.
		

Example usage:
	(*hookable)->J9HookRegisterWithCallSite(hookable, J9HOOK_VM_OBJECT_ALLOCATE_WITHIN_THRESHOLD, eventOccurred, OMR_GET_CALLSITE(), NULL);

	static void
	eventOccurred(J9HookInterface** hookInterface, UDATA eventNum, void* voidData, void* userData)
	{
		J9VMAllocationThresholdEvent* eventData = voidData;
		. . .
	}
 */
#define J9HOOK_VM_OBJECT_ALLOCATE_WITHIN_THRESHOLD 87
typedef struct J9VMAllocationThresholdEvent {
	struct J9VMThread* currentThread;
	j9object_t object;
	UDATA size;
	UDATA lowThreshold;
	UDATA highThreshold;
} J9VMAllocationThresholdEvent;

/* J9HOOK_VM_SLEEP

			Triggered when a thread is about to sleep.
		

Example usage:
	(*hookable)->J9HookRegisterWithCallSite(hookable, J9HOOK_VM_SLEEP, eventOccurred, OMR_GET_CALLSITE(), NULL);

	static void
	eventOccurred(J9HookInterface** hookInterface, UDATA eventNum, void* voidData, void* userData)
	{
		J9VMSleepEvent* eventData = voidData;
		. . .
	}
 */
#define J9HOOK_VM_SLEEP 88
typedef struct J9VMSleepEvent {
	struct J9VMThread * currentThread;
	I_64 millis;
	I_32 nanos;
} J9VMSleepEvent;

/* J9HOOK_VM_SLEPT

			Triggered when a thread returns from a sleep
		

Example usage:
	(*hookable)->J9HookRegisterWithCallSite(hookable, J9HOOK_VM_SLEPT, eventOccurred, OMR_GET_CALLSITE(), NULL);

	static void
	eventOccurred(J9HookInterface** hookInterface, UDATA eventNum, void* voidData, void* userData)
	{
		J9VMSleptEvent* eventData = voidData;
		. . .
	}
 */
#define J9HOOK_VM_SLEPT 89
typedef struct J9VMSleptEvent {
	struct J9VMThread * currentThread;
	I_64 millis;
	I_32 nanos;
	I_64 startTicks;
} J9VMSleptEvent;

/* J9HOOK_VM_CORRUPT_CACHE

			Triggered when VM finds the shared class cache to be corrupt
		

Example usage:
	(*hookable)->J9HookRegisterWithCallSite(hookable, J9HOOK_VM_CORRUPT_CACHE, eventOccurred, OMR_GET_CALLSITE(), NULL);

	static void
	eventOccurred(J9HookInterface** hookInterface, UDATA eventNum, void* voidData, void* userData)
	{
		J9VMCorruptCache* eventData = voidData;
		. . .
	}
 */
#define J9HOOK_VM_CORRUPT_CACHE 90
typedef struct J9VMCorruptCache {
	struct J9VMThread* vmThread;
} J9VMCorruptCache;

/* J9HOOK_VM_CLASSES_REDEFINED

			Triggered after classes are successfully redefined.
			Exclusive VM access is held by the current thread.
		

Example usage:
	(*hookable)->J9HookRegisterWithCallSite(hookable, J9HOOK_VM_CLASSES_REDEFINED, eventOccurred, OMR_GET_CALLSITE(), NULL);

	static void
	eventOccurred(J9HookInterface** hookInterface, UDATA eventNum, void* voidData, void* userData)
	{
		J9VMClassesRedefinedEvent* eventData = voidData;
		. . .
	}
 */
#define J9HOOK_VM_CLASSES_REDEFINED 91
typedef struct J9VMClassesRedefinedEvent {
	struct J9VMThread* currentThread;
} J9VMClassesRedefinedEvent;

/* J9HOOK_VM_CLASS_VERIFICATION_START

			Triggered before starting class verification.
		

Example usage:
	(*hookable)->J9HookRegisterWithCallSite(hookable, J9HOOK_VM_CLASS_VERIFICATION_START, eventOccurred, OMR_GET_CALLSITE(), NULL);

	static void
	eventOccurred(J9HookInterface** hookInterface, UDATA eventNum, void* voidData, void* userData)
	{
		J9VMClassVerificationStartEvent* eventData = voidData;
		. . .
	}
 */
#define J9HOOK_VM_CLASS_VERIFICATION_START 92
typedef struct J9VMClassVerificationStartEvent {
	struct J9BytecodeVerificationData* verifyData;
	BOOLEAN newFormat;
} J9VMClassVerificationStartEvent;

/* J9HOOK_VM_CLASS_VERIFICATION_FALLBACK

			Triggered before starting method verification.
		

Example usage:
	(*hookable)->J9HookRegisterWithCallSite(hookable, J9HOOK_VM_CLASS_VERIFICATION_FALLBACK, eventOccurred, OMR_GET_CALLSITE(), NULL);

	static void
	eventOccurred(J9HookInterface** hookInterface, UDATA eventNum, void* voidData, void* userData)
	{
		J9VMMethodVerificationEndEvent* eventData = voidData;
		. . .
	}
 */
#define J9HOOK_VM_CLASS_VERIFICATION_FALLBACK 93
typedef struct J9VMMethodVerificationEndEvent {
	struct J9BytecodeVerificationData* verifyData;
	BOOLEAN newFormat;
} J9VMMethodVerificationEndEvent;

/* J9HOOK_VM_CLASS_VERIFICATION_END

			Triggered after completed class verification.
		

Example usage:
	(*hookable)->J9HookRegisterWithCallSite(hookable, J9HOOK_VM_CLASS_VERIFICATION_END, eventOccurred, OMR_GET_CALLSITE(), NULL);

	static void
	eventOccurred(J9HookInterface** hookInterface, UDATA eventNum, void* voidData, void* userData)
	{
		J9VMClassVerificationEndEvent* eventData = voidData;
		. . .
	}
 */
#define J9HOOK_VM_CLASS_VERIFICATION_END 94
typedef struct J9VMClassVerificationEndEvent {
	struct J9BytecodeVerificationData* verifyData;
	BOOLEAN newFormat;
} J9VMClassVerificationEndEvent;

/* J9HOOK_VM_METHOD_VERIFICATION_START

			Triggered before starting method verification.
		

Example usage:
	(*hookable)->J9HookRegisterWithCallSite(hookable, J9HOOK_VM_METHOD_VERIFICATION_START, eventOccurred, OMR_GET_CALLSITE(), NULL);

	static void
	eventOccurred(J9HookInterface** hookInterface, UDATA eventNum, void* voidData, void* userData)
	{
		J9VMMethodVerificationStartEvent* eventData = voidData;
		. . .
	}
 */
#define J9HOOK_VM_METHOD_VERIFICATION_START 95
typedef struct J9VMMethodVerificationStartEvent {
	struct J9BytecodeVerificationData* verifyData;
} J9VMMethodVerificationStartEvent;

/* J9HOOK_VM_STACKMAPFRAME_VERIFICATION

			Triggered after completed each stackmap frame verification.
		

Example usage:
	(*hookable)->J9HookRegisterWithCallSite(hookable, J9HOOK_VM_STACKMAPFRAME_VERIFICATION, eventOccurred, OMR_GET_CALLSITE(), NULL);

	static void
	eventOccurred(J9HookInterface** hookInterface, UDATA eventNum, void* voidData, void* userData)
	{
		J9VMStackMapFrameVerificationEvent* eventData = voidData;
		. . .
	}
 */
#define J9HOOK_VM_STACKMAPFRAME_VERIFICATION 96
typedef struct J9VMStackMapFrameVerificationEvent {
	struct J9BytecodeVerificationData* verifyData;
} J9VMStackMapFrameVerificationEvent;

/* J9HOOK_JAVA_BASE_LOADED
Triggered once enough initialization has taken place to use modules

Example usage:
	(*hookable)->J9HookRegisterWithCallSite(hookable, J9HOOK_JAVA_BASE_LOADED, eventOccurred, OMR_GET_CALLSITE(), NULL);

	static void
	eventOccurred(J9HookInterface** hookInterface, UDATA eventNum, void* voidData, void* userData)
	{
		J9VMModuleStartEvent* eventData = voidData;
		. . .
	}
 */
#define J9HOOK_JAVA_BASE_LOADED 97
typedef struct J9VMModuleStartEvent {
	struct J9VMThread* vmThread;
} J9VMModuleStartEvent;

/* J9HOOK_VM_DEEP_IDLE_STATE_ENTERED

			Triggered when JVM entered deep idle state.
		

Example usage:
	(*hookable)->J9HookRegisterWithCallSite(hookable, J9HOOK_VM_DEEP_IDLE_STATE_ENTERED, eventOccurred, OMR_GET_CALLSITE(), NULL);

	static void
	eventOccurred(J9HookInterface** hookInterface, UDATA eventNum, void* voidData, void* userData)
	{
		J9VMDeepIdleStateEnteredEvent* eventData = voidData;
		. . .
	}
 */
#define J9HOOK_VM_DEEP_IDLE_STATE_ENTERED 98
typedef struct J9VMDeepIdleStateEnteredEvent {
	struct J9VMThread* vmThread;
	U_32 level;
} J9VMDeepIdleStateEnteredEvent;

/* J9HOOK_VM_RUNTIME_STATE_CHANGED

			Triggered when JVM's runtime state is changed
		

Example usage:
	(*hookable)->J9HookRegisterWithCallSite(hookable, J9HOOK_VM_RUNTIME_STATE_CHANGED, eventOccurred, OMR_GET_CALLSITE(), NULL);

	static void
	eventOccurred(J9HookInterface** hookInterface, UDATA eventNum, void* voidData, void* userData)
	{
		J9VMRuntimeStateChanged* eventData = voidData;
		. . .
	}
 */
#define J9HOOK_VM_RUNTIME_STATE_CHANGED 99
typedef struct J9VMRuntimeStateChanged {
	struct J9VMThread* vmThread;
	U_32 state;
} J9VMRuntimeStateChanged;

/* J9HOOK_VM_MODULE_LOAD

			Triggered when Java module has been loaded
		

Example usage:
	(*hookable)->J9HookRegisterWithCallSite(hookable, J9HOOK_VM_MODULE_LOAD, eventOccurred, OMR_GET_CALLSITE(), NULL);

	static void
	eventOccurred(J9HookInterface** hookInterface, UDATA eventNum, void* voidData, void* userData)
	{
		J9VMModuleLoadEvent* eventData = voidData;
		. . .
	}
 */
#define J9HOOK_VM_MODULE_LOAD 100
typedef struct J9VMModuleLoadEvent {
	struct J9VMThread* currentThread;
	struct J9Module* module;
} J9VMModuleLoadEvent;

/* J9HOOK_VM_MODULE_UNLOAD

			Triggered when Java module is being unloaded
		

Example usage:
	(*hookable)->J9HookRegisterWithCallSite(hookable, J9HOOK_VM_MODULE_UNLOAD, eventOccurred, OMR_GET_CALLSITE(), NULL);

	static void
	eventOccurred(J9HookInterface** hookInterface, UDATA eventNum, void* voidData, void* userData)
	{
		J9VMModuleUnloadEvent* eventData = voidData;
		. . .
	}
 */
#define J9HOOK_VM_MODULE_UNLOAD 101
typedef struct J9VMModuleUnloadEvent {
	struct J9VMThread* currentThread;
	struct J9Module* module;
} J9VMModuleUnloadEvent;

/* J9HOOK_VM_PREPARING_FOR_CHECKPOINT
Triggered as the JVM is preparing for a checkpoint

Example usage:
	(*hookable)->J9HookRegisterWithCallSite(hookable, J9HOOK_VM_PREPARING_FOR_CHECKPOINT, eventOccurred, OMR_GET_CALLSITE(), NULL);

	static void
	eventOccurred(J9HookInterface** hookInterface, UDATA eventNum, void* voidData, void* userData)
	{
		J9CheckpointEvent* eventData = voidData;
		. . .
	}
 */
#define J9HOOK_VM_PREPARING_FOR_CHECKPOINT 102
typedef struct J9CheckpointEvent {
	struct J9VMThread* currentThread;
} J9CheckpointEvent;

/* J9HOOK_VM_PREPARING_FOR_RESTORE
Triggered as the JVM is preparing to restore from a checkpointed JVM

Example usage:
	(*hookable)->J9HookRegisterWithCallSite(hookable, J9HOOK_VM_PREPARING_FOR_RESTORE, eventOccurred, OMR_GET_CALLSITE(), NULL);

	static void
	eventOccurred(J9HookInterface** hookInterface, UDATA eventNum, void* voidData, void* userData)
	{
		J9RestoreEvent* eventData = voidData;
		. . .
	}
 */
#define J9HOOK_VM_PREPARING_FOR_RESTORE 103
typedef struct J9RestoreEvent {
	struct J9VMThread* currentThread;
} J9RestoreEvent;

/* J9HOOK_VM_VIRTUAL_THREAD_STARTED

			Triggered once the virtual thread is ready to run. At this point the virtual thread has an object associated with it.
		

Example usage:
	(*hookable)->J9HookRegisterWithCallSite(hookable, J9HOOK_VM_VIRTUAL_THREAD_STARTED, eventOccurred, OMR_GET_CALLSITE(), NULL);

	static void
	eventOccurred(J9HookInterface** hookInterface, UDATA eventNum, void* voidData, void* userData)
	{
		J9VirtualThreadStartedEvent* eventData = voidData;
		. . .
	}
 */
#define J9HOOK_VM_VIRTUAL_THREAD_STARTED 104
typedef struct J9VirtualThreadStartedEvent {
	struct J9VMThread* currentThread;
} J9VirtualThreadStartedEvent;

/* J9HOOK_VM_VIRTUAL_THREAD_END

			Triggered when a virtual thread is about to die.
		

Example usage:
	(*hookable)->J9HookRegisterWithCallSite(hookable, J9HOOK_VM_VIRTUAL_THREAD_END, eventOccurred, OMR_GET_CALLSITE(), NULL);

	static void
	eventOccurred(J9HookInterface** hookInterface, UDATA eventNum, void* voidData, void* userData)
	{
		J9VirtualThreadEndEvent* eventData = voidData;
		. . .
	}
 */
#define J9HOOK_VM_VIRTUAL_THREAD_END 105
typedef struct J9VirtualThreadEndEvent {
	struct J9VMThread* currentThread;
} J9VirtualThreadEndEvent;

/* J9HOOK_VM_VIRTUAL_THREAD_MOUNT

			Triggered whenever a virtual thread mounts.
			currentThread->threadObject is the virtual thread at the point of trigger.
		

Example usage:
	(*hookable)->J9HookRegisterWithCallSite(hookable, J9HOOK_VM_VIRTUAL_THREAD_MOUNT, eventOccurred, OMR_GET_CALLSITE(), NULL);

	static void
	eventOccurred(J9HookInterface** hookInterface, UDATA eventNum, void* voidData, void* userData)
	{
		J9VMVirtualThreadMountEvent* eventData = voidData;
		. . .
	}
 */
#define J9HOOK_VM_VIRTUAL_THREAD_MOUNT 106
typedef struct J9VMVirtualThreadMountEvent {
	struct J9VMThread* currentThread;
} J9VMVirtualThreadMountEvent;

/* J9HOOK_VM_VIRTUAL_THREAD_UNMOUNT

			Triggered whenever a virtual thread unmounts.
			currentThread->threadObject is the virtual thread at the point of trigger.
		

Example usage:
	(*hookable)->J9HookRegisterWithCallSite(hookable, J9HOOK_VM_VIRTUAL_THREAD_UNMOUNT, eventOccurred, OMR_GET_CALLSITE(), NULL);

	static void
	eventOccurred(J9HookInterface** hookInterface, UDATA eventNum, void* voidData, void* userData)
	{
		J9VMVirtualThreadUnmountEvent* eventData = voidData;
		. . .
	}
 */
#define J9HOOK_VM_VIRTUAL_THREAD_UNMOUNT 107
typedef struct J9VMVirtualThreadUnmountEvent {
	struct J9VMThread* currentThread;
} J9VMVirtualThreadUnmountEvent;

/* J9HOOK_VM_VIRTUAL_THREAD_DESTROY

			Triggered after J9HOOK_VM_VIRTUAL_THREAD_END to free native resources.
			currentThread->threadObject is the virtual thread at the point of trigger.
		

Example usage:
	(*hookable)->J9HookRegisterWithCallSite(hookable, J9HOOK_VM_VIRTUAL_THREAD_DESTROY, eventOccurred, OMR_GET_CALLSITE(), NULL);

	static void
	eventOccurred(J9HookInterface** hookInterface, UDATA eventNum, void* voidData, void* userData)
	{
		J9VMVirtualThreadDestroyEvent* eventData = voidData;
		. . .
	}
 */
#define J9HOOK_VM_VIRTUAL_THREAD_DESTROY 108
typedef struct J9VMVirtualThreadDestroyEvent {
	struct J9VMThread* currentThread;
} J9VMVirtualThreadDestroyEvent;

/* J9HOOK_VM_CRIU_CHECKPOINT
Triggered at the end of CRIU checkpoint, just before CRIU dump

Example usage:
	(*hookable)->J9HookRegisterWithCallSite(hookable, J9HOOK_VM_CRIU_CHECKPOINT, eventOccurred, OMR_GET_CALLSITE(), NULL);

	static void
	eventOccurred(J9HookInterface** hookInterface, UDATA eventNum, void* voidData, void* userData)
	{
		J9VMCRIUCheckpointEvent* eventData = voidData;
		. . .
	}
 */
#define J9HOOK_VM_CRIU_CHECKPOINT 109
typedef struct J9VMCRIUCheckpointEvent {
	struct J9VMThread* vmThread;
} J9VMCRIUCheckpointEvent;

/* J9HOOK_VM_CRIU_RESTORE
Triggered at the beginning of CRIU restore, just after runInternalJVMRestoreHooks()

Example usage:
	(*hookable)->J9HookRegisterWithCallSite(hookable, J9HOOK_VM_CRIU_RESTORE, eventOccurred, OMR_GET_CALLSITE(), NULL);

	static void
	eventOccurred(J9HookInterface** hookInterface, UDATA eventNum, void* voidData, void* userData)
	{
		J9VMCRIURestoreEvent* eventData = voidData;
		. . .
	}
 */
#define J9HOOK_VM_CRIU_RESTORE 110
typedef struct J9VMCRIURestoreEvent {
	struct J9VMThread* vmThread;
} J9VMCRIURestoreEvent;

#endif /* VMHOOK_H */
